@@ -37,3 +37,26 @@ nc <- nc %>% mutate(frac74 = SID74 / BIR74)
 plot(a[2], col = c(grey(.8), grey(.5)))
 plot(st_geometry(Ashe), border = '#ff8888', add = TRUE, lwd = 2)
 
+## ------------------------------------------------------------------------
+x = st_sf(a = 1:2, geom = st_sfc(st_point(c(0,0)), st_point(c(1,1))))
+y = data.frame(a = 2:3)
+merge(x, y)
+merge(x, y, all = TRUE)
+right_join(x, y)
+
+## ----fig=TRUE------------------------------------------------------------
+x = st_sf(a = 1:3, geom = st_sfc(st_point(c(1,1)), st_point(c(2,2)), st_point(c(3,3))))
+y = st_buffer(x, 0.1)
+x = x[1:2,]
+y = y[2:3,]
+plot(st_geometry(x), xlim = c(.5, 3.5))
+plot(st_geometry(y), add = TRUE)
+
+## ------------------------------------------------------------------------
+st_join(x, y)
+st_join(y, x)
+
+## ------------------------------------------------------------------------
+st_join(x, y, join = st_covers) # no matching y records: points don't cover circles
+st_join(y, x, join = st_covers) # matches for those circles covering a point
+
